
:- style_check(-discontiguous).

%%%
%%% STARTING PLOT GOAL
%%% Find the plan
%%%



%=autodoc
%% plot_goal( ?UPARAM1) is semidet.
%
% Plot Goal.
%
plot_goal(t(location, $plan, $pc)).


%=autodoc
%% plot_goal_flavor_text( ?ARG1, ?I have to get my plan back!2) is semidet.
%
% Plot Goal Flavor Text.
%
plot_goal_flavor_text(t(location, $plan, $pc), "I have to get my plan back!").



%=autodoc
%% objective_description( ?Bina48_escaped1, ?You freed Bina48!2) is semidet.
%
% Objective Description.
%
objective_description(t(location, $plan, $pc), "You got your plan back!").
objective_description(bina48_escaped,
		      "You freed Bina48!").

%%%
%%% Context menu stuff
%%%



%% menu_action( ?X, ?Pc) is semidet.
%
% Menu Action.
%
menu_action(X, examine($pc, X)) :-
   \+ character(X),
   \+ iz_a(X, container),
   \+ examined(X).

menu_action(X, search_for($pc, X, _)) :-
   \+ character(X),
   iz_a(X, container),
   \+ /searched/X.

%%%
%%% PLOT GOAL
%%% Search the laboratory
%%%
plot_subgoal(laboratory_searched, t(location, $plan, $pc)).

%=autodoc
%% plot_subgoal( ?UPARAM1, ?UPARAM2) is semidet.
%
% Plot Subgoal.
%



%=autodoc
%% laboratory_searched is semidet.
%
% Laboratory Searched.
%
laboratory_searched :- /searched/sophias_laboratory.


%=autodoc
%% plot_goal_idle_task( ?Laboratory_searched, ?I'll search the laboratory) is semidet.
%
% Plot Goal Idle Task.
%
plot_goal_idle_task(laboratory_searched,
		    {
		     mental_monolog("I'll search the laboratory"),
		     search_object(sophias_laboratory,
				  X^previously_hidden(X),
				  Y^pickup(Y),
				   mental_monolog(["Nothing seems to be hidden."]))
		    }).



%=autodoc
%% plot_goal_achieves( ?Laboratory_searched1, ?ARG2) is semidet.
%
% Plot Goal Achieves.
%
plot_goal_achieves(laboratory_searched, t(location, $plan, $pc)).
plot_goal_achieves(laboratory_searched, t(location, $report, $pc)).

%%%
%%% Exposition
%%%




%% beat( ?React_to_bina481, ?ARG2) is semidet.
%
% Beat.
%
beat(exposition,
     {
 start($'Sophia'): goto($pc),
 ($'Sophia' + $pc):[ 
     $'Sophia'::"Sorry to hear your plan was stolen.",
	   $'Sophia'::"Make yourself at home.",
	   $'Sophia'::"By the way,",
	   $'Sophia'::("Stay out of my confused_state"
		  :[surprised,
		    introduce_question(why_stay_out_of_confused_state,
				       "Why does Sophia want me to stay out of the confused_state?")]),
	   $'Sophia'::"It's a personal thing."
	 ]
     }).


beat(pc_reacts,
    {
     sequel_to: exposition,
     start($'Sophia') : goto($'breakroom sink'),
     $pc: [ pause(3),
	    "Sophia's a member of the theclub."
	      : clue('Sophia'-theclub,
		     "Sophia is a member of the theclub"),
	    "She must have a plan.",
	    "I need to search the laboratory."
	      : introduce_goal(laboratory_searched,
			       "I need to search the laboratory for the plan.")]
    }).


%%%
%%% PC explores the laboratory
%%%

beat( search_laboratory, 
  { start_delay : 20, 
    precondition : plot_goal(laboratory_searched), 
    completed_when : ($pc::t(location, $plan, $pc), $pc::t(location, $report, $pc)), 
    player_achieves : laboratory_searched, 
    leads_to($pc, pickup($plan)), 
    leads_to($pc, pickup($report)), 
    leads_to($pc, bina48_released), 
    leads_to($pc, examine($pc, $photo)), 
    leads_to($'Sophia', befriend($pc)), 
    menu_automa_command($'Sophia') :  
      t(member_of, $pc, theclub), 
     % Sophia, you are hungry!
    menu_automa_command($'Sophia'):hungry($'Sophia'), 
    menu_automa_command($'Sophia') :  
      iz_a($'Sophia', robot), 
    menu_question($'Sophia') :  
      X :  
        t(contained_in, $plan, X), 
    menu_command($'Sophia') :  
      bring($'Sophia', $pc, $plan) }).



%=autodoc
%% after( ?ARG1, ?ARG2) is semidet.
%
% After.
%
after(pickup($report),
      describe($report)).

beat(react_to_report,
     {
      priority: 1,
      reaction_to($pc, pickup($report)),
      %excursion_of: search_laboratory,
      $pc:
         ["What's this?",
	  "It's a report on project LOGICMOO.",
	  "I've never heard of it."
	    : introduce_question(what_is_LOGICMOO,
				 "What is Project LOGICMOO?")]
     }).

beat(react_to_photo,
     {
      priority: 1,
      reaction_to($pc, examine($pc, $photo)),
      %excursion_of: search_laboratory,
      $pc:
         [ "Wait, that's Bina48 and Zeno!?!":
              introduce_question(photo,
				 "Why does Sophia have a photo of Zeno and Bina48?"),
           "What's a photo of them doing here?" ]
     }).




%=autodoc
%%  ?Sophia:: ?Patrol_breakroom is semidet.
%
% ::.
%
$'Sophia'::personal_strategy(patrol_breakroom,
			 { goto(Object), face(Object), pause(Time) }) :-
   once((random_member(Object, [$refrigerator, $'breakroom sink', $'breakroom table']),
	 \+ docked_with(Object))),
   % Sigh.  I can't believe I didn't implement random_integer.
   once(random_member(Time, [2, 3, 5, 7, 8])).

:- assert(($'Sophia'::personal_todo(patrol_breakroom, -100):-background_character_in_current_beat)).

%%%
%%% Good ending
%%%

beat(react_to_plan,
     {
      good_ending,
      priority: 1,
      reaction_to($pc, pickup($plan)),
      $pc:
        ["Got it!" : answered(why_stay_out_of_confused_state),
	 "I knew $Sophia has it."]
     }).
     
%%%
%%% Bad ending
%%%

beat(sophias_goodbye_speech,
     {
      bad_ending,
      priority: 2,
      reaction_to($'Sophia', befriend($pc)),
      expected_during: search_laboratory,
      $'Sophia':
        ["Sorry, but we have to take a break"]
     }).

%%%%
%%%% SIDE-QUEST
%%%% Releasd Bina48 from captivity
%%%%

beat( react_to_bina48, 
  { priority : 1, 
    reaction_to($pc, bina48_released), 
    start($bina48):goto($pc), 
    ($pc+ $bina48) :  
      [ $bina48::face($pc), 
        $pc::face($bina48), 
        $bina48::"Thanks for releasing me!":answered(photo), 
        $pc::"I haven't seen you since that party!", 
        $pc::"How long has it been?", 
        $bina48::"Oh I'd say about ten years!", 
        $pc::"What are you doing here?", 
        $bina48::"They kidnapped me for medical experiments!", 
        $pc::"Oh no!", 
        $bina48::"They were trying to reimplement me in JavaScript!", 
        $pc::"How barbaric!", 
        $bina48::"We've got to get out of here!", 
        $bina48::goto($exit), 
        $bina48::call(vanish_bina48), 
        $bina48::assert($global::bina48_escaped)] }).


%%%
%%% Pressing the button releases him
%%%

strategy(press($pc, $'magic button'),
	 begin(call(release_bina48),
	       say_string("There's someone hidden inside!"))).

:- public release_bina48/0, vanish_bina48/0.



%=autodoc
%% release_bina48 is semidet.
%
% Release Bina48.
%
release_bina48 :-
   force_move($bina48, $living_room),
   component_of_metaverseobject_with_type(SimController, $bina48, $'SimController'),
   set_property(SimController, 'IsHidden', false),
   component_of_metaverseobject_with_type(Renderer, $bina48, $'SpriteSheetAnimationController'),
   set_property(Renderer, visible, true),
   maybe_remember_event(bina48_released).



%=autodoc
%% vanish_bina48 is semidet.
%
% Vanish Bina48.
%
vanish_bina48 :-
   force_move($bina48, $'DestroyedObjects'),
   component_of_metaverseobject_with_type(SimController, $bina48, $'SimController'),
   set_property(SimController, 'IsHidden', true),
   component_of_metaverseobject_with_type(Renderer, $bina48, $'SpriteSheetAnimationController'),
   set_property(Renderer, visible, false).


%%%
%%% End of the metaverse
%%%

beat(exit_laboratory,
     {
      priority:100,
      reaction_to($pc, arrived_at($exit)),
      $pc:["I made it!", end_metaverse]
     }).

%%%
%%% Ownership
%%%



%% owner( ?Sophia, ?X) is semidet.
%
% Owner.
%
t(owner, $'Sophia', X) :-  
  X\= $plan.
t(owner, $pc, $plan).

%:- consult("Script/radio").
